/*
* aureport-output.c - Print the report
* Copyright (c) 2005 Red Hat Inc., Durham, North Carolina.
* All Rights Reserved. 
*
* This software may be freely redistributed and/or modified under the
* terms of the GNU General Public License as published by the Free
* Software Foundation; either version 2, or (at your option) any
* later version.
*
* This program is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU General Public License for more details.
*
* You should have received a copy of the GNU General Public License
* along with this program; see the file COPYING. If not, write to the
* Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*
* Authors:
*   Steve Grubb <sgrubb@redhat.com>
*/

#include "config.h"
#include <stdio.h>
#include <string.h>
#include "aureport-scan.h"
#include "aureport-options.h"
#include "ausearch-lookup.h"

static void do_summary_output(void);
unsigned int line_item;


void print_title(void)
{
	line_item = 0U;
	printf("\n");
	switch (report_type)
        {
		case RPT_SUMMARY:
			printf("Summary Report\n");
			printf("==============\n");
			break;
		case RPT_AVC:
			printf("AVC Summary Report\n");
			printf("====================================\n");
			printf("# comm subj syscall action obj event\n");
			printf("====================================\n");
			break;
		case RPT_CONFIG:
			printf("Config Change Summary Report\n");
			printf("=========================\n");
//			printf("# auid action what success event\n");
			printf("# auid success date event\n");
			printf("=========================\n");
			break;
		case RPT_LOGIN:
			printf("Login Summary Report\n");
			printf("=======================================\n");
			printf("# auid host term exe success date event\n");
			printf("=======================================\n");
			break;
		case RPT_ACCT_MOD:
			printf("Account Modifications Summary Report\n");
			printf("====================================\n");
			printf("# auid addr term exe success event\n");
			printf("====================================\n");
			break;
		case RPT_TIME:
			printf("Log Time Range Report\n");
			printf("=====================\n");
			break;
		case RPT_EVENT:
			if (report_detail == D_DETAILED) {
				printf("Event Summary Report\n");
				printf("===========================\n");
				printf("# event type auid date time\n");
				printf("===========================\n");
			} else {
				printf("Specific Event Report\n");
				printf("=====================\n");
			}
			break;
		case RPT_FILE:
			if (report_detail == D_DETAILED) {
			    printf("File Summary Report\n");
			    printf("=====================================\n");
			    printf("# file syscall success exe auid event\n");
			    printf("=====================================\n");
			} else {
				printf("Specific File Report\n");
				printf("====================\n");
			}
			break;
		case RPT_HOST:
			if (report_detail == D_DETAILED) {
				printf("Host Summary Report\n");
				printf("=========================\n");
				printf("# host syscall auid event\n");
				printf("=========================\n");
			} else {
				printf("Specific Host Report\n");
				printf("====================\n");
			}
			break;
		case RPT_PID:
			if (report_detail == D_DETAILED) {
				printf("Process ID Summary Report\n");
				printf("============================\n");
				printf("# pid exe syscall auid event\n");
				printf("============================\n");
			} else {
				printf("Specific Process ID Report\n");
				printf("==========================\n");
			}
			break;
		case RPT_SYSCALL:
			if (report_detail == D_DETAILED) {
				printf("Syscall Summary Report\n");
				printf("=============================\n");
				printf("# syscall pid comm auid event\n");
				printf("=============================\n");
			} else {
				printf("Specific Syscall Report\n");
				printf("=======================\n");
			}
			break;
		case RPT_TERM:
			if (report_detail == D_DETAILED) {
				printf("Terminal Summary Report\n");
				printf("==========================\n");
				printf("# term host exe auid event\n");
				printf("==========================\n");
			} else {
				printf("Specific Terminal Report\n");
				printf("========================\n");
			}
			break;
		case RPT_USER:
			if (report_detail == D_DETAILED) {
				printf("User ID Summary Report\n");
				printf("==========================\n");
				printf("# auid term host exe event\n");
				printf("==========================\n");
			} else {
				printf("Specific User ID Report\n");
				printf("=======================\n");
			}
			break;
		case RPT_WATCH:
			if (report_detail == D_DETAILED) {
			    printf("Watch Summary Report\n");
			    printf(
				"==========================================\n");
			    printf(
				"# watch key syscall success exe auid event\n");
			    printf(
				"==========================================\n");
			} else {
				printf("Specific Watch Report\n");
				printf("=====================\n");
			}
			break;
		case RPT_EXE:
			if (report_detail == D_DETAILED) {
				printf("Executable ID Summary Report\n");
				printf("============================\n");
				printf("# exe term host auid event\n");
				printf("============================\n");
			} else {
				printf("Specific Executable Report\n");
				printf("==========================\n");
			}
			break;
		case RPT_ANOMALY:
			if (report_detail == D_DETAILED) {
				printf("Anomaly Summary Report\n");
				printf("===============================\n");
				printf("# type exe term host auid event\n");
				printf("===============================\n");
			} else {
				printf("Specific Anomaly Report\n");
				printf("=======================\n");
			}
			break;
		case RPT_RESPONSE:
			if (report_detail == D_DETAILED) {
				printf("Response to Anomaly Summary Report\n");
				printf("==================================\n");
				printf("# type success event\n");
				printf("==================================\n");
			} else {
				printf("Specific Response to Anomaly Report\n");
				printf("===================================\n");
			}
			break;
		default:
			break;
	}
}

void print_per_event_item(llist *l)
{
	char buf[128];
	char name[64];
	char date[32];
	struct tm *tv;

	line_item++;
	switch (report_type)
	{
		case RPT_AVC:
			// line command subject syscall action obj res event
			printf("%u. %s %s %s %s %s %s %lu\n", 
				line_item,
				l->s.comm ? l->s.comm : "?",
				l->s.scontext, 
				aulookup_syscall(l, buf,sizeof(buf)),
				l->s.avc_action,
				l->s.tcontext, aulookup_result(l->s.avc_result),
				l->e.serial);
			break;
		case RPT_CONFIG:
			// FIXME:who, action, what, outcome, event
			// NOW: auid success date event
			tv = localtime(&l->e.sec);
			strftime(date, sizeof(date), "%x", tv);
			printf("%u. %s success %s %lu\n",
				line_item,
				aulookup_uid(l->s.loginuid, name, sizeof(name)),
				date, l->e.serial);
			break;
		case RPT_LOGIN:
			// who, addr, terminal, exe, success, date,event
			// Special note...uid is used here because that is
			// the way that the message works. This is because
			// on failed logins, loginuid is not set.
			tv = localtime(&l->e.sec);
			strftime(date, sizeof(date), "%x", tv);
			printf("%u. %s %s %s %s %s %s %lu\n", line_item,
				aulookup_uid(l->s.uid, name, sizeof(name)),
				l->s.hostname, l->s.terminal,
				l->s.exe, aulookup_success(l->s.success),
				date, l->e.serial);
			break;
		case RPT_ACCT_MOD:
			// who, addr, terminal, exe, success, event
			printf("%u. %s %s %s %s %s %lu\n", line_item,
				aulookup_uid(l->s.loginuid, name, sizeof(name)),
				l->s.hostname ? l->s.hostname : "?",
				l->s.terminal ? l->s.terminal : "?",
				l->s.exe ? l->s.exe : "?",
				aulookup_success(l->s.success),
				l->e.serial);
			break;
		case RPT_EVENT:	// report_detail == D_DETAILED
			//        event, type, who, date
			tv = localtime(&l->e.sec);
			strftime(date, sizeof(date), "%x %X", tv);
			printf("%u. %lu %s %s %s\n", line_item,
				l->e.serial,
				audit_msg_type_to_name(l->head->type),
				aulookup_uid(l->s.loginuid, name, sizeof(name)),
				date);
			break;
		case RPT_FILE:	// report_detail == D_DETAILED
			// file, syscall, success, exe, who, event
			slist_first(l->s.filename);
			printf("%u. %s %s %s %s %s %lu\n", line_item,
				l->s.filename->cur->str,
				aulookup_syscall(l,buf,sizeof(buf)),
				aulookup_success(l->s.success),
				l->s.exe ? l->s.exe : "?",
				aulookup_uid(l->s.loginuid, name, sizeof(name)),
				l->e.serial);
			break;
		case RPT_HOST:	// report_detail == D_DETAILED
			// host, syscall, who, event
			printf("%u. %s %s %s %lu\n", line_item,
				l->s.hostname,
				aulookup_syscall(l,buf,sizeof(buf)),
				aulookup_uid(l->s.loginuid, name, sizeof(name)),
				l->e.serial);
			break;
		case RPT_PID:	// report_detail == D_DETAILED
			// pid, exe, syscall, who, event
			printf("%u. %u %s %s %s %lu\n", line_item,
				l->s.pid,
				l->s.exe ? l->s.exe : "?",
				aulookup_syscall(l,buf,sizeof(buf)),
				aulookup_uid(l->s.loginuid, name, sizeof(name)),
				l->e.serial);
			break;
		case RPT_SYSCALL:	// report_detail == D_DETAILED
			// syscall, pid, comm, who, event
			printf("%u. %s %u %s %s %lu\n", line_item,
				aulookup_syscall(l,buf,sizeof(buf)),
				l->s.pid,
				l->s.comm ? l->s.comm : "?",
				aulookup_uid(l->s.loginuid, name, sizeof(name)),
				l->e.serial);
			break;
		case RPT_TERM:	// report_detail == D_DETAILED
			// terminal, host, exe, who, event
			printf("%u. %s %s %s %s %lu\n", line_item,
				l->s.terminal, l->s.hostname,
				l->s.exe,
				aulookup_uid(l->s.loginuid, name, sizeof(name)),
				l->e.serial);
			break;
		case RPT_USER:	// report_detail == D_DETAILED
			// who, terminal, host, exe, event
			printf("%u. %s %s %s %s %lu\n", line_item,
				aulookup_uid(l->s.loginuid, name, sizeof(name)),
				l->s.terminal ? l->s.terminal : "?",
				l->s.hostname ? l->s.hostname : "?",
				l->s.exe ? l->s.exe : "?",
				l->e.serial);
			break;
		case RPT_WATCH:	// report_detail == D_DETAILED
			// watch, syscall, who, event
			slist_first(l->s.filename);
			printf("%u. %s %s %s %s %s %s %lu\n", line_item,
				l->s.filename->cur->str,
				l->s.filename->cur->key ? 
					l->s.filename->cur->key : "<none>",
				aulookup_syscall(l,buf,sizeof(buf)),
				aulookup_success(l->s.success),
                                l->s.exe ? l->s.exe : "?",
				aulookup_uid(l->s.loginuid, name, sizeof(name)),
				l->e.serial);
			break;
		case RPT_EXE:	// report_detail == D_DETAILED
			// exe, terminal, host, who, event
			printf("%u. %s %s %s %s %lu\n", line_item,
				l->s.exe ? l->s.exe : "?", 
				l->s.terminal ? l->s.terminal : "?",
				l->s.hostname ? l->s.hostname : "?",
				aulookup_uid(l->s.loginuid, name, sizeof(name)),
				l->e.serial);
			break;
		case RPT_ANOMALY:	// report_detail == D_DETAILED
			// type exe term host auid event
			printf("%u. %s %s %s %s %s %lu\n", line_item,
				audit_msg_type_to_name(l->head->type),
				l->s.exe ? l->s.exe : "?", 
				l->s.terminal ? l->s.terminal : "?",
				l->s.hostname ? l->s.hostname : "?",
				aulookup_uid(l->s.loginuid, name, sizeof(name)),
				l->e.serial);
			break;
		case RPT_RESPONSE:	// report_detail == D_DETAILED
			// type success event
			printf("%u. %s %s %lu\n", line_item,
				audit_msg_type_to_name(l->head->type),
				aulookup_success(l->s.success),
				l->e.serial);
			break;
		default:
			break;
	}
}

void print_wrap_up(void)
{
	switch (report_type)
	{
		case RPT_SUMMARY:
			do_summary_output();
			break;
		default:
			break;
	}
}

static void do_summary_output(void)
{
	extern event very_first_event;
	extern event very_last_event;

	printf("Range of time: ");
	{
		struct tm *btm;
		char tmp[32];

		btm = localtime(&very_first_event.sec);
		strftime(tmp, sizeof(tmp), "%x %T", btm);
		printf("%s.%03d - ", tmp, very_first_event.milli);
		btm = localtime(&very_last_event.sec);
		strftime(tmp, sizeof(tmp), "%x %T", btm);
		printf("%s.%03d\n", tmp, very_last_event.milli);
	}
	printf("Number of changes in configuration: %lu\n", sd.changes);
	printf("Number of changes to accounts or groups: %lu\n",
							 sd.acct_changes);
	printf("Number of logins: %lu\n", sd.good_logins);
	printf("Number of failed logins: %lu\n", sd.bad_logins);
	printf("Number of users: %u\n", sd.users.cnt);
	printf("Number of terminals: %u\n", sd.terms.cnt);
	printf("Number of host names: %u\n", sd.hosts.cnt);
	printf("Number of executables: %u\n", sd.exes.cnt);
	printf("Number of files: %u\n", sd.files.cnt);
	printf("Number of avc denials: %lu\n", sd.avcs);
	printf("Number of failed syscalls: %lu\n", sd.failed_syscalls);
	printf("Number of watched file events: %lu\n", sd.fs_watches);
	printf("Number of anomaly events: %lu\n", sd.anomalies);
	printf("Number of responses to anomaly events: %lu\n", sd.responses);
	printf("Number of process IDs: %u\n", sd.pids.cnt);
	printf("Number of events: %lu\n", sd.events);
	printf("\n");
}

